@@ -21,11 +21,13 @@ from pathlib import Path
 from tqdm import tqdm
 from tabulate import tabulate
 from decimal import Decimal
+# from numba import njit
 
 from pgmpy.models import BayesianNetwork, NaiveBayes
 from pgmpy.factors.discrete import TabularCPD
 from pgmpy.sampling import BayesianModelSampling, GibbsSampling
 from pgmpy import readwrite
+from pgmpy.metrics import structure_score
 
 from ismember import ismember
 import pypickle
@@ -87,6 +89,7 @@ def make_DAG(DAG, CPD=None, methodtype='bayes', checkmodel=True, verbose=3):
     methodtype : str (default: 'bayes')
         * 'bayes': Bayesian model
         * 'nb' or 'naivebayes': Special case of Bayesian Model where the only edges in the model are from the feature variables to the dependent variable. Or in other words, each tuple should start with the same variable name such as: edges = [('A', 'B'), ('A', 'C'), ('A', 'D')]
+        * 'markov': Markov model
     checkmodel : bool
         Check the validity of the model. The default is True
     verbose : int, optional
@@ -106,7 +109,7 @@ def make_DAG(DAG, CPD=None, methodtype='bayes', checkmodel=True, verbose=3):
     >>> import bnlearn as bn
     >>> edges = [('A', 'B'), ('A', 'C'), ('A', 'D')]
     >>> DAG = bn.make_DAG(edges, methodtype='naivebayes')
-    >>> bn.plot(DAG)
+    >>> fig = bn.plot(DAG)
 
     """
     if (CPD is not None) and (not isinstance(CPD, list)):
@@ -810,11 +813,11 @@ def get_node_properties(model, node_color='#1f456e', node_size=None, verbose=3):
     >>> node_properties['A']['node_size']=2000
     >>> node_properties['A']['node_color']='#000000'
     >>> # Make plot
-    >>> bn.plot(model, interactive=False, node_properties=node_properties)
+    >>> fig = bn.plot(model, interactive=False, node_properties=node_properties)
     >>>
     >>> # Example: Specify all nodes
     >>> node_properties = bn.get_node_properties(model, node_size=1000, node_color='#000000')
-    >>> bn.plot(model, interactive=False, node_properties=node_properties)
+    >>> fig = bn.plot(model, interactive=False, node_properties=node_properties)
 
     """
     # https://networkx.org/documentation/networkx-1.7/reference/generated/networkx.drawing.nx_pylab.draw_networkx_nodes.html
@@ -870,7 +873,7 @@ def get_edge_properties(model, color='#000000', weight=1, minscale=1, maxscale=1
     >>> edge_properties[('A', 'B')]['weight']=10
     >>> edge_properties[('A', 'B')]['color']='#8A0707'
     >>> # Make plot
-    >>> bn.plot(model, interactive=False, edge_properties=edge_properties)
+    >>> fig = bn.plot(model, interactive=False, edge_properties=edge_properties)
     >>>
     >>> # Example 2:
     >>>  # Load asia DAG
@@ -884,7 +887,7 @@ def get_edge_properties(model, color='#000000', weight=1, minscale=1, maxscale=1
     >>> # Make adjustments
     >>> edge_properties[('tub', 'either')]['color']='#8A0707'
     >>> # Make plot
-    >>> bn.plot(model, interactive=True, edge_properties=edge_properties)
+    >>> fig = bn.plot(model, interactive=True, edge_properties=edge_properties)
 
     """
     # https://networkx.org/documentation/networkx-1.7/reference/generated/networkx.drawing.nx_pylab.draw_networkx_nodes.html
@@ -931,8 +934,8 @@ def plot(model,
          node_size=None,
          node_properties=None,
          edge_properties=None,
-         params_interactive={'width': '70%', 'height': '800px', 'notebook': False, 'layout': None, 'font_color': False, 'bgcolor': '#ffffff'},
-         params_static={'minscale': 1, 'maxscale': 10, 'figsize': (15, 10), 'width': None, 'height': None, 'font_size': 14, 'font_family': 'sans-serif', 'alpha': 0.8, 'node_shape': 'o', 'layout': 'spring_layout', 'font_color': '#000000', 'facecolor': 'white', 'edge_alpha': 0.8, 'arrowstyle': '-|>', 'arrowsize': 30},
+         params_interactive={'width': '70%', 'height': '800px', 'notebook': False, 'layout': None, 'font_color': False, 'bgcolor': '#ffffff', 'select_menu': True, 'filter_menu': True, 'cdn_resources': 'remote'},
+         params_static={'minscale': 1, 'maxscale': 10, 'figsize': (15, 10), 'width': None, 'height': None, 'font_size': 14, 'font_family': 'sans-serif', 'alpha': 0.8, 'node_shape': 'o', 'layout': 'spring_layout', 'font_color': '#000000', 'facecolor': 'white', 'edge_alpha': 0.8, 'arrowstyle': '-|>', 'arrowsize': 30, 'visible': True},
          verbose=3):
     """Plot the learned stucture.
 
@@ -991,13 +994,13 @@ def plot(model,
     >>> model = bn.structure_learning.fit(df)
     >>>
     >>> # plot static
-    >>> G = bn.plot(model)
+    >>> fig = bn.plot(model)
     >>>
     >>> # plot interactive
-    >>> G = bn.plot(model, interactive=True)
+    >>> fig = bn.plot(model, interactive=True)
     >>>
     >>> # plot interactive with various settings
-    >>> bn.plot(model, interactive=True, node_color='#8A0707', node_size=35, params_interactive = {'height':'800px', 'width':'70%', 'bgcolor':'#0f0f0f0f'})
+    >>> fig = bn.plot(model, interactive=True, node_color='#8A0707', node_size=35, params_interactive = {'height':'800px', 'width':'70%', 'bgcolor':'#0f0f0f0f'})
     >>>
     >>> # plot with node properties
     >>> node_properties = bn.get_node_properties(model)
@@ -1005,26 +1008,27 @@ def plot(model,
     >>> node_properties['xray']['node_color']='#8A0707'
     >>> node_properties['xray']['node_size']=50
     >>> # Plot
-    >>> bn.plot(model, interactive=True, node_properties=node_properties)
+    >>> fig = bn.plot(model, interactive=True, node_properties=node_properties)
     >>>
 
     """
+    ax = None
     # Check whether edges are available
     if model['adjmat'].sum().sum()==0:
         if verbose>=3: print('[bnlearn]> Nothing to plot because no edges are present between nodes. ')
         return None
 
     # Plot properties
-    defaults = {'height': '800px', 'width': '70%', 'notebook': False, 'layout': None, 'font_color': False, 'bgcolor': '#ffffff', 'directed': True}
+    defaults = {'height': '800px', 'width': '70%', 'notebook': False, 'layout': None, 'font_color': False, 'bgcolor': '#ffffff', 'directed': True, 'cdn_resources': 'remote', 'select_menu': True, 'filter_menu': True}
     params_interactive = {**defaults, **params_interactive}
-    defaults = {'minscale': 1, 'maxscale': 10, 'figsize': (15, 10), 'height': None, 'width': None, 'font_size': 14, 'font_family': 'sans-serif', 'alpha': 0.8, 'layout': 'spring_layout', 'font_color': 'k', 'facecolor': '#ffffff', 'node_shape': 'o', 'edge_alpha': 0.8, 'arrowstyle': '-|>', 'arrowsize': 30}
+    defaults = {'minscale': 1, 'maxscale': 10, 'figsize': (15, 10), 'height': None, 'width': None, 'font_size': 14, 'font_family': 'sans-serif', 'alpha': 0.8, 'layout': 'spring_layout', 'font_color': 'k', 'facecolor': '#ffffff', 'node_shape': 'o', 'edge_alpha': 0.8, 'arrowstyle': '-|>', 'arrowsize': 30, 'visible': True}
     params_static = {**defaults, **params_static}
 
-    ##### DEPRECATED IN LATER VERSION #####
+    # DEPRECATED IN LATER VERSION
     if (params_static.get('width') is not None) or (params_static.get('height') is not None):
         # if verbose>=2: print('[bnlearn]> Warning: [height] and [width] will be removed in further version. Please use: params_static={"figsize": (15, 10)}.')
         params_static['figsize'] = (15 if params_static['width'] is None else params_static['width'], 10 if params_static['height'] is None else params_static['height'])
-    ##### END BLOCK #####
+    # END BLOCK
 
     out = {}
     G = nx.DiGraph()  # Directed graph
@@ -1071,24 +1075,25 @@ def plot(model,
     # Plot
     if interactive:
         # Make interactive plot
-        _plot_interactive(model, params_interactive, nodelist, node_colors, node_sizes, edgelist, edge_colors, edge_weights, title, verbose=verbose)
+        ax = _plot_interactive(model, params_interactive, nodelist, node_colors, node_sizes, edgelist, edge_colors, edge_weights, title, verbose=verbose)
     else:
         # Make static plot
-        _plot_static(model, params_static, nodelist, node_colors, node_sizes, G, pos, edge_colors, edge_weights)
+        ax = _plot_static(model, params_static, nodelist, node_colors, node_sizes, G, pos, edge_colors, edge_weights, visible=params_static['visible'])
 
     # Store
+    out['ax']=ax
     out['pos']=pos
     out['G']=G
     out['node_properties']=node_properties
     out['edge_properties']=edge_properties
-    return(out)
+    return out
 
 
 # %% Plot interactive
 # def _plot_static(model, params_static, nodelist, node_colors, node_sizes, title, verbose=3):
-def _plot_static(model, params_static, nodelist, node_colors, node_sizes, G, pos, edge_colors, edge_weights):
+def _plot_static(model, params_static, nodelist, node_colors, node_sizes, G, pos, edge_colors, edge_weights, visible=True):
     # Bootup figure
-    plt.figure(figsize=params_static['figsize'], facecolor=params_static['facecolor'])
+    ax = plt.figure(figsize=params_static['figsize'], facecolor=params_static['facecolor'])
     # nodes
     nx.draw_networkx_nodes(G, pos, nodelist=nodelist, node_size=node_sizes, alpha=params_static['alpha'], node_color=node_colors, node_shape=params_static['node_shape'])
     # edges
@@ -1099,13 +1104,20 @@ def _plot_static(model, params_static, nodelist, node_colors, node_sizes, G, pos
     # Plot text of the weights
     # nx.draw_networkx_edge_labels(G, pos, edge_labels=nx.get_edge_attributes(G, 'weight'), font_color=params_static['font_color'])
     # Making figure nice
-    ax = plt.gca()
-    ax.set_axis_off()
-    plt.show()
+    # ax = plt.gca()
+    # ax.set_axis_off()
+    ax.set_visible(visible)
+    return ax
 
 
 # %% Plot interactive
 def _plot_interactive(model, params_interactive, nodelist, node_colors, node_sizes, edgelist, edge_colors, edge_weights, title, verbose=3):
+    # If not notebook
+    if not params_interactive['notebook']:
+        # https://pyvis.readthedocs.io/en/latest/tutorial.html?highlight=cdn_resources#using-pyvis-within-jupyter-notebook
+        params_interactive['cdn_resources']='local'
+
+    # Try to import pyvis
     try:
         from pyvis.network import Network
         from IPython.core.display import display, HTML
@@ -1133,9 +1145,10 @@ def _plot_interactive(model, params_interactive, nodelist, node_colors, node_siz
     g.show_buttons(filter_=['physics'])
     # Display
     filename = title.strip().replace(' ', '_') + '.html'
-    g.show(filename)
+    g.show(filename, local=params_interactive['notebook'])
     display(HTML(filename))
     # webbrowser.open('bnlearn.html')
+    return os.path.abspath(filename)
 
 
 # %% Plot properties
@@ -1327,7 +1340,7 @@ def import_DAG(filepath='sprinkler', CPD=True, checkmodel=True, verbose=3):
     --------
     >>> import bnlearn as bn
     >>> model = bn.import_DAG('sprinkler')
-    >>> bn.plot(model)
+    >>> fig = bn.plot(model)
 
     """
     PATH_TO_DATA = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')
@@ -1436,6 +1449,7 @@ def _filter_df(adjmat, df, verbose=3):
 
 
 # %% Make prediction in inference model
+# @njit
 def predict(model, df, variables, to_df=True, method='max', verbose=3):
     """Predict on data from a Bayesian network.
 
@@ -1723,3 +1737,81 @@ def _normalize_weights(weights, minscale=1, maxscale=10):
     from sklearn.preprocessing import MinMaxScaler
     weights = MinMaxScaler(feature_range=(minscale, maxscale)).fit_transform(weights).flatten()
     return(weights)
+
+
+# %% Compute structure scores.
+def structure_scores(model, df, scoring_method=['k2', 'bds', 'bic', 'bdeu'], verbose=3, **kwargs):
+    """Compute structure scores.
+
+    Description
+    -----------
+    Each model can be scored based on its structure. However, the score doesn't have very straight forward
+    interpretebility but can be used to compare different models. A higher score represents a better fit.
+    This method only needs the model structure to compute the score. The structure score functionality
+    can be found here: :func:`bnlearn.bnlearn.structure_scores`.
+
+    Parameters
+    ----------
+    model: The bnlearn instance such as pgmpy.base.DAG or pgmpy.models.BayesianNetwork
+        The model whose score needs to be computed.
+
+    df: pd.DataFrame instance
+        The dataset against which to score the model.
+
+    scoring_method: str ( k2 | bdeu | bds | bic )
+        The following four scoring methods are supported currently: 1) K2Score
+        2) BDeuScore 3) BDsScore 4) BicScore
+
+    kwargs: kwargs
+        Any additional parameters parameters that needs to be passed to the
+        scoring method.
+
+    Returns
+    -------
+    Model score: float
+        A score value for the model.
+
+    Examples
+    --------
+    >>> import bnlearn as bn
+    >>> # Load example dataset
+    >>>
+    >>> df = bn.import_example('sprinkler')
+    >>> edges = [('Cloudy', 'Sprinkler'), ('Cloudy', 'Rain'), ('Sprinkler', 'Wet_Grass'), ('Rain', 'Wet_Grass')]
+    >>>
+    >>> # Make the Bayesian DAG
+    >>> DAG = bn.make_DAG(edges)
+    >>> model = bn.parameter_learning.fit(DAG, df)
+    >>>
+    >>> # Structure scores are stored in the model dictionary.
+    >>> model['structure_scores']
+    >>>
+    >>> # Compute the structure score for as specific scoring-method.
+    >>> bn.structure_scores(model, df, scoring_method="bic")
+    """
+    method = None
+    show_message = True
+    scores = {}
+    # Get models and method
+    if isinstance(model, dict):
+        method = model.get('config')['method']
+        model = model.get('model', None)
+    if isinstance(scoring_method, str): scoring_method = [scoring_method]
+    if verbose>=3: print('[bnlearn] >Compute structure scores %s for model comparison (higher is better).' %(scoring_method))
+
+    # Return if method not supported
+    if np.any(np.isin(method, ['cs', 'constraintsearch'])):
+        if verbose>=2: print('[bnlearn] >Warning: Structure scoring could not be computed. Method [%s] not supported.' %(method))
+        return scores
+
+    # Compute structure scores
+    if model is not None:
+        for s in scoring_method:
+            try:
+                scores[s] = structure_score(model, df, scoring_method=s)
+            except:
+                if verbose>=2 and show_message:
+                    print('[bnlearn] >Warning: Structure scoring could not be computed. DataFrame issues (?)')
+                    show_message=False
+    # Return
+    return scores
