@@ -5,6 +5,95 @@ from pgmpy.models import BayesianNetwork, NaiveBayes
 from pgmpy.estimators import ExhaustiveSearch, HillClimbSearch, TreeSearch
 from pgmpy.factors.discrete import TabularCPD
 
+
+# %% Notebook example
+# Example dataframe sprinkler_data.csv can be loaded with: 
+import bnlearn as bn
+df = bn.import_example()
+# df = pd.read_csv('sprinkler_data.csv')
+model = bn.structure_learning.fit(df)
+
+
+# Set some colors to the edges and nodes
+node_properties = bn.get_node_properties(model)
+node_properties['Sprinkler']['node_color']='#FF0000'
+
+edge_properties = bn.get_edge_properties(model)
+edge_properties[('Rain', 'Cloudy')]['color']='#FF0000'
+edge_properties[('Rain', 'Cloudy')]['weight']=5
+
+G = bn.plot(model,
+            node_properties=node_properties,
+            edge_properties=edge_properties,
+            interactive=True,
+            params_interactive={'notebook': False, 'filter_menu': True, 'select_menu': True},
+            )
+
+# G = bn.plot(model, interactive=True, params_interactive={'notebook': True}, node_properties=node_properties, edge_properties=edge_properties)
+
+# %%
+import bnlearn as bn
+model = bn.import_DAG('asia')
+
+# Make single inference
+query = bn.inference.fit(model, variables=['lung', 'bronc', 'xray'], evidence={'smoke':1})
+print(query)
+print(bn.query2df(query))
+
+# Lets create an example dataset with 100 samples and make inferences on the entire dataset.
+df = bn.sampling(model, n=10000)
+
+# Each sample will be assesed and the states with highest probability are returned.
+Pout = bn.predict(model, df, variables=['lung'])
+
+print(Pout)
+#     Cloudy  Rain         p
+# 0        0     0  0.647249
+# 1        0     0  0.604230
+# ..     ...   ...       ...
+# 998      0     0  0.604230
+# 999      1     1  0.878049
+
+# %%
+import bnlearn as bn
+model = bn.import_DAG('asia', CPD=True)
+CPDs = bn.print_CPD(model)
+CPDs.keys()
+# dict_keys(['asia', 'bronc', 'dysp', 'either', 'lung', 'smoke', 'tub', 'xray'])
+print(CPDs['smoke'])
+#    smoke    p
+# 0      0  0.5
+# 1      1  0.5
+
+# %% Issue 65: return (fig, ax)
+import bnlearn as bn
+model = bn.import_DAG('sprinkler', CPD=True)
+df = bn.sampling(model, n=1000, methodtype='bayes')
+fig = bn.plot(model, params_static={'visible': True})
+
+fig2 = bn.plot(model, interactive=True)
+
+
+# %% Issue Mail:
+# from pgmpy.models import BayesianNetwork
+
+# # create instance
+# hd_bn = BayesianNetwork()
+
+# # create dag
+# hd_bn.add_edges_from(hd_edges)
+
+# # structure score
+# print(f'K2 score: {structure_score(hd_bn,bn_df,scoring_method="k2")}')
+# print(f'BDeu score: {metrics.structure_score(hd_bn,bn_df,scoring_method="bdeu")}')
+# print(f'BDS score: {metrics.structure_score(hd_bn,bn_df,scoring_method="bds")}')
+# print(f'BIC score: {metrics.structure_score(hd_bn,bn_df,scoring_method="bic")}')
+
+# K2 score: -3469.9237414747818
+# BDeu score: -3501.3510316331763
+# BDS score: -3579.8380555853423
+# BIC score: -3616.8520164722863
+
 # %% Issue #60: Floating Point Errors.
 import bnlearn as bn
 
@@ -41,7 +130,6 @@ model = bn.structure_learning.fit(df)
 G = bn.plot(model)
 
 # %% Issue MAIL: Store CPDs after printing.
-
 import bnlearn as bn
 
 # Load example dataset
@@ -1083,7 +1171,7 @@ cpd_E = TabularCPD(variable='E', variable_card=2,
 print(cpd_E)
 
 
-DAG = bn.make_DAG(DAG, CPD=cpd_A, checkmodel=True)
+DAG = bn.make_DAG(DAG, CPD=cpd_A, checkmodel=False)
 bn.print_CPD(DAG, checkmodel=True)
 
 # %% Create a simple DAG:
