@@ -21,7 +21,6 @@ Currently, the library supports parameter learning for *discrete* nodes:
 
 # %% Libraries
 from pgmpy.estimators import BayesianEstimator
-# from pgmpy.estimators import MaximumLikelihoodEstimator
 import bnlearn
 import warnings
 warnings.filterwarnings("ignore")
@@ -153,5 +152,6 @@ def fit(model, df, methodtype='bayes', scoretype='bdeu', smooth=None, n_jobs=-1,
     out['adjmat'] = adjmat
     out['config'] = config
     out['model_edges'] = list(model.edges())
+    out['structure_scores'] = bnlearn.structure_scores(out, df)
 
     return(out)
